# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .guardrail_name import GuardrailName
from .guardrail_result import GuardrailResult
from .json_node import JsonNode


class Guardrail(UniversalBaseModel):
    id: typing.Optional[str] = None
    entity_id: str
    secondary_id: str
    project_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    If null, the default project is used
    """

    name: GuardrailName
    result: GuardrailResult
    config: JsonNode
    details: JsonNode

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
