# This file was auto-generated by Fern from our API Definition.

import typing

JsonListString = typing.Union[
    typing.Dict[str, typing.Optional[typing.Any]], typing.List[typing.Dict[str, typing.Optional[typing.Any]]], str
]
